// C++ Dependencies
using namespace std;
# include <iostream>
# include "Env.h"  
# include <string>
# include "GocrAlgorithms.h"  // remove when we start using dlsym.

// C Dependencies
//  - JOB is a global variable currently needed by the C code. Will be
//    removed soon.
Job* JOB = NULL;

namespace SomeName {
  // ******************************************************************
  // CONSTRUCTORS

  Env::Env(int argc, const char* argv[]) {
    // Initialize the dirty C state.
    // - The 'Job' type, and the JOB global var, are crucial to the
    //   proper execution of the gocr code. However, many fields
    //   within 'job' are not initialized until 'processArgs'
    //   has been called.  It is for this reason that 'argc' and
    //   'argv' are passed in to Env - the class is not fully
    //   initialized unless Job is fully initialized.
    if ( JOB == NULL ) { Initialize(); }
    Job * job = JOB;
    job_init(job);
    this->_job = job;

    // Initialize "good" state to default values.
    this->verboseIs(0);

    // Initialize state from command-line options. This can initialize
    // both low-level 'job' state, and high-level Env state.
    this->processArgs(argc, argv);

    // Create an instance of the appopriate StrategyFactory subclass -
    // this class dictates which strategy to use for every algorithm,
    // and is fundamental to the flexibility provided by the SomeName
    // framework.
    //
    // IMPORTANT: This must be generalized, twice:
    //   1) A flag added specifying the class.  For now, we can
    //      hard-code tests against the string to determine which
    //      class to create.
    //   2) The flag value should be used to dynamically load and
    //      create an instance of the appropriate class!  That way
    //      the client does not need to modify this code in order
    //      to get a new factory to work.  FUTURE FIX: This should
    //      happen soon!
    StrategyFactory* algorithms = NULL;
    const string fstr = "GocrAlgorithms";
    if ( fstr == "GocrAlgorithms" ) {
      algorithms = new GocrAlgorithms();
      // IMPORTANT: Must call init - the constructor has not fully
      // initialized itself!
      algorithms->init();
    } else {
      throw string("UnknownFactoryException: ") + fstr;
    }
    this->algorithmsIs(algorithms);
   }

  void Env::Initialize() {
    JOB = new Job;
    job_init(JOB);
  }

  // ******************************************************************
  // ACCESSORS
  

  // ******************************************************************
  // METHODS

  /// Add a new Page for file 'file'.
  ///
  /// Creates a Page instances, makes it the current page within
  /// this Env, adds it to the 'pages' vector, and returns it.
  Page* Env::registerPage(const string& file) {
    Page* page = new Page(this, file);
    this->currentIs(page);
    this->pagesRef().push_back(page);
    return page;
  }

  /// Initialize various fields based on command-line flags.
  bool Env::processArgs(int argc, const char** argv) {
    // NOTE: We cannot use the 'process_args' method from 'gocr'
    // because 'process_args' isn't part of the lib (it is in 'gocr.c'
    // which defines the 'main' method). We don't want to be using the
    // 'gocr' code anyway, so this limitation is fine.

    int i;
    const char *s1;
    Job* job = this->jobRef();
    assert(job);
    if (argc <= 1) {
      //out_version(1);
      exit(0);
    }
#ifdef HAVE_PGM_H
    pnm_init(&argc, &argv);
#endif

    bool result = false;
    job->src.fname = NULL;
    /* process arguments */
    for (i = 1; i < argc; i++) {
      if (strcmp(argv[i], "--help") == 0) {
        result = false;
        goto EndArgs;
      }
      if (argv[i][0] == '-' && argv[i][1] != 0) {
        s1 = "";
        if (i + 1 < argc)
          s1 = argv[i + 1];
        switch (argv[i][1]) {
          case 'h': /* help */
            result = false;
            goto EndArgs;
            break;
          case 'i': /* input image file */
            job->src.fname = const_cast<char*>(s1); // cast forced upon us by C code.
            i++;
            break;
          case 'e': /* logging file */
            if (s1[0] == '-' && s1[1] == '\0') {
#ifdef HAVE_UNISTD_H
              dup2(STDOUT_FILENO, STDERR_FILENO); /* -e /dev/stdout  works */
#else
              fprintf(stderr, "stderr redirection not possible without unistd.h\n");
#endif           
            }
            else if (!freopen(s1, "w", stderr)) {
              fprintf(stderr, "stderr redirection to %s failed\n", s1);
            }
            i++;
            break;
          case 'p': /* database path */
            job->cfg.db_path= const_cast<char*>(s1); // cast forced upon us by C code.
            i++;
            break;
          case 'o': 
            // Specifies where identified characters should be written.
            // FUTURE FIX: We definitely do NOT want to be redirecting
            // stdout like this.  There is no need to be dumping text
            // to (redirected) stdout when we can just open a write
            // stream on the desired file.
            this->outfileIs(s1);
            i++;
            break;
          case 'f': /* output format */
            if (strcmp(s1, "ISO8859_1") == 0) job->cfg.out_format=ISO8859_1; else
              if (strcmp(s1, "TeX")       == 0) job->cfg.out_format=TeX; else
                if (strcmp(s1, "HTML")      == 0) job->cfg.out_format=HTML; else
                  if (strcmp(s1, "XML")       == 0) job->cfg.out_format=XML; else
                    if (strcmp(s1, "SGML")      == 0) job->cfg.out_format=SGML; else
                      if (strcmp(s1, "UTF8")      == 0) job->cfg.out_format=UTF8; else
                        if (strcmp(s1, "ASCII")     == 0) job->cfg.out_format=ASCII; else
                          fprintf(stderr,"Warning: unknown format (-f %s)\n",s1);
            i++;
            break;
          case 'c': /* list of chars (_ = not recognized chars) */
            job->cfg.lc = const_cast<char*>(s1); // cast forced upon us by C code.
            i++;
            break;
          case 'C': /* char filter, default: NULL (all chars) */
            /* ToDo: UTF8 input, wchar */
            job->cfg.cfilter = const_cast<char*>(s1); // cast forced upon us by C code.
            i++;
            break;
          case 'd': /* dust size */
            job->cfg.dust_size = atoi(s1);
            i++;
            break;
          case 'l': /* grey level 0<160<=255, 0 for autodetect */
            job->cfg.cs = atoi(s1);
            i++;
            break;
          case 's': /* spacewidth/dots (0 = autodetect) */
            job->cfg.spc = atoi(s1);
            i++;
            break;
          case 'v': /* verbose mode */
            job->cfg.verbose |= atoi(s1);
            i++;
            break;
          case 'V': /* verbose mode */
            this->verboseIs( atoi(s1) );
            i++;
            break;
          case 'm': /* operation modes */
            job->cfg.mode |= atoi(s1);
            i++;
            break;
          case 'n': /* numbers only */
            job->cfg.only_numbers = atoi(s1);
            i++;
            break;
          case 'x': /* initialize progress output s1=fname */
            cerr << "ERORR: -x currently disabled in C++ implementation" << endl;
            // ini_progress(s1);
            i++;
            break;
          case 'a': /* set certainty */
            job->cfg.certainty = atoi(s1);;
            i++;
            break;
          default:
            fprintf(stderr, "# unknown option use -h for help\n");
        }
        continue;
      }
      else /* argument can be filename v0.2.5 */ if (argv[i][0] != '-'
                                                     || argv[i][1] == '\0' ) {
        job->src.fname = const_cast<char*>(argv[i]); // cast forced upon us by C code.
      }
    }

    EndArgs:
    if ( job->src.fname ) {
      // If we've established an input file, we automatically create a
      // Page for it.
      this->registerPage(job->src.fname);
    }
    return result;
  }
  
  void Env::wrapup() {
    // cerr << "NOTE: Finished" << endl;

  }

  void Env::test() {
    cerr << "ERROR: Not unit test for Env - FIX THIS!" << endl;
  }

};
