# ifndef Env_h
# define Env_h Env_h

// C++ Dependencies
# include "Root.h"            // parent
# include <vector>             // uses
# include "Page.h"            // has-a  (needed here to define friends)

// C Dependencies (temporary)
# include "gocr.h"  // job_t, job_init

// Declaration
namespace SomeName {

  class StrategyFactory;       // has-a
    
  // ###################################################################
  /// \class Env
  ///
  /// \brief The public front-end to the SomeName set of classes
  ///
  /// This class acts as a front-end, maintaining all user-provided and
  /// internal meta-information. A 'main' program can create an instance
  /// of this class and interact entirely with it, with or without
  /// involving the other classes in SomeName.
  // ###################################################################

  class Env : public Root {
  public:
    // ################
    // Public Interface
    Env(int argc, const char* argv[]);

    /// Initialize fields that control variations in processing
    /// algorithm.
    bool processArgs(int argc, const char** argv);

    /// Add a new Page to my list of pages
    Page* registerPage(const std::string& file);

    /// Returns the currently "active" page.
    Page* currentPage() { return this->currentRef(); }

    /// Perform any final post-processing mandated by config fields.
    void wrapup();

    /// Perform class initializaton.
    ///
    /// This method is currently important because the GOCR
    /// code has a global variable that must be initialized in
    /// order for any gocr code to work.  Once the global var
    /// is removed, this method may no longer be needed.
    ///
    /// This method is called in Env::Env, but must also be called in
    /// any unit-testing method that doesn't use Env directly (the
    /// global JOB variable must still be initialized if the class in
    /// question uses any gocr code).
    static void Initialize();

    // ########################
    // Encapsulation Violations
    //  - Temporarily, Page needs access to non-const version
    //    of same Job instances stored in Env, and thus needs
    //    Env::jobRef()
    friend Page::Page(Env* env, const std::string& file);

    /// Unit testing method.
    ///
    /// This static method should create instances of the class (and
    /// instances of any other class necessary) and perform tests to
    /// ensure that all methods within the class are working as
    /// expected.
    static void test();

    // #########
    // Accessors
  public:
    inline const Page* current() const { return this->_current; }
    inline const std::vector<Page*> & pages() const { return this->_pages; }
    inline const Job* job() const { return this->_job; }

    inline const int verbose() const { return this->_verbose; }
    inline const std::string & outfile() const { return this->_outfile; }

    inline const StrategyFactory* algorithms() const { return this->_algorithms; }

  protected:
    inline void currentIs(Page* current) { this->_current = current; }
    inline void pagesIs(const std::vector<Page*> & pages) { this->_pages = pages; }
    inline void jobIs(Job* job) { this->_job = job; }

    inline void verboseIs(int verbose) { this->_verbose = verbose; }
    inline void outfileIs(const std::string & outfile) { this->_outfile = outfile; }

  private: 
    inline Page*& currentRef() { return this->_current; }
    inline std::vector<Page*>& pagesRef() { return this->_pages; }
    inline Job*& jobRef() { return this->_job; }

    inline int& verboseRef() { return this->_verbose; }
    inline std::string &   outfileRef() { return this->_outfile; }

    /// An instance of some subclass of StrategyFactory, it
    /// establishes the set of algorithms to use.
    ///
    /// The StrategyFactory interface defines one Factory Method for
    /// each algorithm in the SomeName framework. The 'algorithms'
    /// method creates an instance of a specific subclass in the
    /// appropriate Strategy hierarchy. Every algorithm has an
    /// associated Strategy hierarchy.
    StrategyFactory* _algorithms;
    inline void algorithmsIs(StrategyFactory* algorithms) { this->_algorithms = algorithms; }
    inline StrategyFactory* & algorithmsRef() { return this->_algorithms; }

    /// The file to write output to.  Specified via the -i flag.
    std::string _outfile;

    /// Current page being processed.
    Page* _current; 
    /// List of all pages in environment.  
    ///
    /// Often, there is only one page, but support exists for
    /// for dealing with multiple pages.
    std::vector<Page*> _pages;   
    // Temporary field - not placed in doxygen documentation.
    Job* _job;

    /// The level of verbosity specified for this execution.
    ///
    /// A value of 0 means no diagnostic output is printed.
    /// A value of 1 means major diagnostic items are noted.
    /// Increasing values provide more and more detailed diagnostics.
    int _verbose;


  };
};

// C++ Dependencies
# include "Page.h" // has-a

# endif // Env_h

