# ifndef Glyph_h
# define Glyph_h Glyph_h

// C Dependencies
//  - FUTURE FIX: these headers are only needed while the C++ interface
//    relies on the underlying C code.

# include "pnm.h"
# include "gocr.h"  // Box

// C++ Dependencies
# include "Element.h"

namespace SomeName {

  // ###################################################################
  /// \class Glyph
  ///
  /// \brief A rectangular region of the overall image that has been
  ///  identified as representing a single character.
  ///
  /// The Box class represents all information associated with the
  /// graphical reprsentation of a to-be-identified character.  The
  /// correctness of this box (i.e. whether it really does represent
  /// a single character) depends on:
  ///  - the accuracy of line detection algorithms (currently pretty good)
  ///  - the accuracy of box detecting algorithms (currently ok, 
  ///    improvements possible)
  ///
  /// Ths class is responsible for converting an image of a character
  /// into a unicode character value, and as such is at the heart of the
  /// OCR code.
  // ###################################################################

  class Glyph : public Element {
  public:
    // ########
    // Methods 
    
    // The constructor.  Parent allowed to be NULL only until
    // dependence on C code is removed.  The 'internal' param
    // is also temporary (it is a Box struct if provided).
    Glyph(Element* parent = NULL, void* internal = NULL);
    
    // Non-standard copy constructor (may switch to pass-by-ref later)
    //  - arg intentionally non-const due to parent-link issues
    Glyph(Glyph* glyph);
    
    // Elemential deallocation
    int reset_box_ac();
    
    // FUTURE FIX: This method is unnecessary in C++ - can just use
    // 'delete'.  Replace all occurences of 
    //     rec->free_box();
    // with
    //     delete rec;
    int free_box();
    
    int merge_boxes(Glyph *glyph);
    
    // ...
    int reduce_vectors(int mode);
    
    // ...
    int cut_box();
    
    // measure mean thickness (so we can detect "large" characters)
    int mean_thickness();

    /// Returns the height (in pixels) of this glyph
    inline int height() const { 
      // FUTURE FIX: The 'y0' and 'y1' fields are carry overs
      // from the C code, but a much cleaner structure will be
      // provided in which 'width' and 'height' are fields
      // inherited from Element.
      return this->box()->y1 - this->box()->y0;
    }

    /// Vertical position from top of page to top of this glyph, in
    /// pixels (indexing from 0)
    inline int top()      const   { 
      return this->box()->y0;
    }

    /// Vertical position from top of page to bottom of this glyph, in
    /// pixels (indexing from 0)
    inline int bottom()   const   { 
      return this->box()->y1;
    }

    /// Vertical position from top of page to highest portion of this
    /// glyph (the ascender), in pixels (indexing from 0)
    inline int ascentAbs() const  {
      return this->box()->m1;
    }

    /// Vertical position from top of page to the x-height portion of
    /// this glyph, in pixels (indexing from 0)
    inline int xheightAbs() const {
      return this->box()->m2;
    }

    /// Vertical position from top of page to baseline of this glyph,
    /// in pixels (indexing from 0)
    inline int baseline() const   {
      return this->box()->m3;
    }

    /// Vertical position from top of page to lowest portion of this
    /// glyph (the descender), in pixels (indexing from 0)
    inline int descentAbs() const {
      return this->box()->m4;
    }

    /// Allows Image instances to be converted to Glyph instances
    /// safely without requiring explicit down-casting.
    virtual Glyph* asGlyph() { return this; }
    virtual const Glyph* asGlyph() const { return this; }

    // Accessors
    virtual int type() const { return Type; }
    inline const Box* box() const { return this->_box; }
    
    virtual void printSummary(std::ostream& os = std::cerr, const std::string& indent = "", int index = -1) const;

    /// Unit testing method.
    ///
    /// This static method should create instances of the class (and
    /// instances of any other class necessary) and perform tests to
    /// ensure that all methods within the class are working as
    /// expected.
    static void test();

  protected:
    // ########
    // Accessors
    inline void            boxIs(Box* box) { this->_box = box; }
    inline Box* &          boxRef() { return this->_box; }
    
    // ########
    // Methods 
    
  private: 
    // ########
    // Accessors
    
    // ########
    // Methods 
    
    // ########
    // State
    Box*            _box;
    static const int Type = 5;
  };
};

# endif // Glyph_h

