
# ifndef Image_h
# define Image_h Image_h

// C++ Dependencies
# include <string>    // uses
# include "Coord.h"  // Coord, u1
# include "Page.h"   // needs friendship for special Image constructor

// C Dependencies
# include "pnm.h"    // struct pixmap

namespace SomeName {
  // ###################################################################
  /// \class Image
  ///
  /// \brief The in-memory representation of the graphical image being
  /// processed.
  ///
  /// Although arbitrary graphical formats will be supported, all
  /// formats will be converted to pnm, and the internal memory
  /// representation will be pnm-based (mostly pgm, greyscale).
  // ###################################################################
  class Image : public Root {

    // IMPORTANT: This class is an early candidate for disassociation from
    // underlying GOCR C-code reliance.  Instead of using the GOCR routines,
    // it will be straight-forward to write our own, or use a universally
    // available library.  We can rely on the existence of the 'convert'
    // executable to perform conversions from arbitrary formats to pnm,
    // and provide support here for reading and writing pnm files.

    // FUTURE FIX: The 'verbose' flag to the various methods is dirty.
    // Would a static Verbose field be better?  Or do we formalize a
    // Config structure that can be used by many classes, and have
    // 'verbose' be one of the fields in this struct?  Passing an optional
    // Config pointer seems much cleaner than just passing around a boolean
    // value.  Code of the form:
    //    pgmWrite("file.pbm", true);
    // just isn't readable, although admittedly
    //    pgmWrite("file.pbm", verbose);
    // is more so.  But 
    //    pgmWrite("file.pbm", config);
    // is more general.

  private:
    /// Temporary default constructor - will be removed!!!
    ///
    /// Initializes state from global JOB variable.  Although not
    /// strictly necessary, it allows us to avoid creating two
    /// 'pix' data-structures for every Page (one for the Job, and
    /// one for the Image).  This constructor makes the Image pixmap
    /// be the same as the Job pixmap.
    Image(bool verbose = false);
    friend Page::Page(Env* env, const std::string& file);

  public:
    // Constructors/Destructors
    Image(const std::string& file, bool verbose = false);

    /// Create a copy of a sub-region of another Image.
    ///
    /// 'topleft' and 'bottomright' identify the pixel boundaries of
    /// the desired region, and are relative to <0,0> in the top left
    /// corner of 'src'.
    ///
    /// Magnification allows the image to be enlarged. Since this is
    /// meant to show the exact pixels, this is intentionally not an
    /// intelligent resizing with anti-aliasing, etc. Instead, every
    /// pixel is repeated 'magnification' times in the output, each
    /// repeat having exactly the same value. Only integral multiples
    /// are allowed.
    Image(const Image* src, const Coord& topleft, const Coord& bottomright, int magnification = 1, bool verbose = false);

    ~Image();
    
    // Accessors
    inline const pix* data() const { return this->_data; }

    /// Obtain the pixel value for position <x,y>
    inline u1 get(u2 x, u2 y) const { return this->_data->p[this->index(x,y)]; }
    inline u1 get(const Coord& pos) const  { return this->_data->p[this->index(pos.x,pos.y)]; }

    inline void set(u2 x, u2 y, int val)       { this->_data->p[this->index(x,y)] = val; }
    inline void set(const Coord& pos, int val) { this->_data->p[this->index(pos.x,pos.y)] = val; }
    
    // Input/Output
    
    // Miscellaneous
    bool writeMono(const std::string& filebase, bool verbose = false) const;
    bool writeGrey(const std::string& filebase, bool verbose = false) const;
    bool writeColor(const std::string& filebase, bool verbose = false) const;

    /// Images can be stored either monochrome, greyscale or color.
    /// The latter uses 3 bytes per pixel, the former two use 1 byte
    /// per pixel.  FUTURE FIX: I assume pbm packs 8 into one byte??
    ///
    /// FUTURE FIX: Should we not bother with color at all, and always
    /// read images in greyscale? Problem with this is that color may
    /// provide useful hints for partitioning and maybe even
    /// identification. For now, we maintain support for color,
    /// although the primary focus currently is on greyscale analysis.
    int  bytesPerPixel() const { return this->_data->bpp; }

    /// Unit testing method.
    ///
    /// This static method should create instances of the class (and
    /// instances of any other class necessary) and perform tests to
    /// ensure that all methods within the class are working as
    /// expected.
    static void test();


  protected:
    // Accessors
    inline void            dataIs(pix* data) { this->_data = data; }
    
    // Methods 

    /// Returns the one-dimensional index associated with point <x,y>
    inline int index(u2 x, u2 y) const { return x + y*this->_data->x; }
    
  private: 
    // Accessors
    inline pix*          dataRef() { return this->_data; }
    
    // Methods 
    
    // State
    pix*           _data;
  };
};

# endif // Image_h

