# ifndef Page_h
# define Page_h Page_h

// C Dependencies (temporary)
# include "gocr.h"    // Job
# include "pcx.h"     // readpcx
# include "pnm.h"     // readpgm
# include "pgm2asc.h" // pgm2asc, getTextLine

// C++ Dependencies
# include <iostream>
# include "Element.h"

// Declaration
namespace SomeName {

  // C++ Dependencies
  class Env;             // belongs-to
  class StrategyFactory; // returns

  // ###################################################################
  /// \class Page
  ///
  /// \brief An Image and associated meta-data representing an entire
  /// page of to-be-scanned data.
  ///
  /// This class is one of the most important in the Part hierarchy. It
  /// may be sub-divided into Regions, Lines, Words and/or Glyphs,
  /// although only Glyphs are crucial.  It plays a special role relative
  /// to other Part subclasses:
  ///  - it is the only Part that does not have a containing parent.
  ///  - it is the only Part to maintain an Image
  ///  - all other Parts are contained within a Page, and can eventually
  ///    obtain a Page (and thus an Image) by following their 'parent'
  ///    pointer.
  /// See the documentation in Part for more details on this.
  ///
  /// Much of the high-level segmenting functionality is defined on
  /// this class, and can be redefined by subclasses. Segmenting
  /// includes region detection, picture detection, noise removal, line
  /// detection, glyph detection, etc. etc., but does not involve
  /// glyph-to-character identification (see Glyph for the
  /// identification interface).
  ///
  /// Each Page is associated with a file (representing the input image
  /// to be analyzed) and an Image (describing the in-memory
  /// representation of the file).
  // ###################################################################

  class Page : public Element {
    // For now, Env gets to set certain fields of Page that violate
    // total encapsulation. FUTURE FIX: This 'friend' statement needs
    // to be refined to a list of methods within Env - the current
    // statement is unnecessarily permissive. And if possible, the
    // friendship should be removed entirely.
    friend class Env;

    // In order to allow all Parts access to an image stored only in
    // Page, we must allow Part::image access to Page::imageRef()
    friend Image* Element::associatedImage() const;

  public:
    // ******************************************************  
    // Constructors/Destructors
    Page(Env* env, const std::string& file);
 
    // ******************************************************  
    // Accessors
    inline const Env* env() const { return this->_env; }
    inline const std::string & file() const { return this->_file; }
    inline const Job* job() const { return this->_job; }
    inline const Image*    image() const { return this->_image; }
 
    // ******************************************************  
    // Miscellaneous

    /// Perform a full analysis: dust removal, line detection,
    /// character detection, character recognition, etc.
    virtual bool process();

    /// Divide the Page into Glyphs.
    /// 
    /// Depending on configuration values, the Glyphs may also be
    /// grouped into Words and/or Lines and/or Regions within the
    /// Page.
    /// 
    /// Returns false if no appropriate content was found, true
    /// otherwise.
    virtual bool segment();

    /// Perform identification of each individual Glyph.
    ///
    /// The collection of Glyphs may be modified during this process.
    /// For example, if the segmenting algorithm mis-identified a
    /// region as containing a character, but recognition suggests it
    /// is more than one character, or only part of a character, then
    /// the Glyph in question may be split or merged as appropriate.
    virtual bool identify();

    /// Write identified text to outfile.
    ///
    /// This method is only applicable after 'identify' has been
    /// invoked, and is responsible any final grouping of Glyphs into
    /// Words, Words into Lines, Lines into Regions, and Regions into
    /// Pages, although such groupings may (optionally) have been
    /// performed during 'segment' or 'identify' as well.
    ///
    /// The output 
    virtual bool format();

    /// Allows Image instances to be converted to Page instances
    /// safely without requiring explicit down-casting.
    virtual Page* asPage() { return this; }
    virtual const Page* asPage() const { return this; }

    /// Print out information about this Part.
    virtual void printSummary(std::ostream& os = std::cerr, const std::string& indent = "", int index = -1) const;

    /// Unit testing method.
    ///
    /// This static method should create instances of the class (and
    /// instances of any other class necessary) and perform tests to
    /// ensure that all methods within the class are working as
    /// expected.
    static void test();

  protected:
    // ******************************************************  
    // Accessors
    inline void envIs(Env* env) { this->_env = env; }
    inline void fileIs(const std::string & file) { this->_file = file; }
    inline void jobIs(Job* job) { this->_job = job; }
    virtual int type() const    { return Type; }
    inline void imageIs(Image* image) { this->_image = image; }
 
    // ******************************************************
    // Methods 
    StrategyFactory* algorithms() const;
    
  private: 
    // ******************************************************
    // Accessors
    inline Env* &          envRef()  { return this->_env;  }
    inline std::string &   fileRef() { return this->_file; }
    inline Job* &          jobRef()  { return this->_job;  }
    inline Image* &        imageRef() { return this->_image; }
    
    // ******************************************************
    // Methods 
    
    // ******************************************************
    // State
    static const int Type = 1;

    /// The Env information.
    ///
    /// Note that although we could treat Env as a Singleton and
    /// avoid storing it here, that will potentially pose problems
    /// for multi-threaded apps and parallel/distributed support
    /// (consider this further!).
    ///
    /// All subclasses of Part need access to Env, but instead of
    /// defining an 'Env' field in every subclass, we simply define a
    /// 'env' method in Part that searches up the parent link until a
    /// Page is found, then asks the Page to return the Env. This is
    /// inefficient, but it is assumed that Env will only be needed
    /// rarely. If this becomes an efficiency bottleneck, we can
    /// consider other solutions (Singleton, Env in each Part, etc.).
    Env*            _env;

    /// The input file to be processed.
    ///
    /// This file is also used to establish a prefix for various
    /// output files.
    std::string     _file;

    /// Temporary: For now, we have each Page store a Job, but
    /// eventually we will want Page to BE (part of) Job. More
    /// specifically the Job struct currently contains four sub-structs
    /// (src, tmp, res, and cfg). The Env class should take over the
    /// 'cfg' data, and the fields from the other three structs should
    /// be placed into this class.
    ///
    /// Note that this field is intentionally not documented in doxygen.
    Job*            _job;

    /// The Image associated with this Page.
    ///
    /// Every Page has an Image, storing the in-memory information
    /// from a graphic-format file. The Image class defines low-level
    /// functionality like pixel get/set, pixel filtering, etc., as
    /// well as Image copying, Image magnification, and Image
    /// write-to-file functionality.
    Image*          _image;

  };
};

// C++ Dependencies
# include "Env.h"    // belongs-to

# endif // Page_h

