# ifndef Root_h
# define Root_h Root_h

/// \mainpage SomeName
/// 
/// This document provides an overall design discussion of SomeName.
/// explaining and justifying the architecture, providing an overview of
/// the purpose of each class, the most important parts of the public
/// interface of each class, etc. Much more detailed documentation is
/// available on a per-class basis within the classes themselves, and via
/// the doxygen pages in 'docs/'.
/// 
/// <hr>
/// \section hierarchy Basic Class Hierarchy
///
/// \verbatim
//       
//                                            Root
//                                              |
//       .---------------------------------------^-----------.
//       |                                       |           |
//       |                                      Element       Env
//       |                                       |   
//       |               .-----------------------^-----------------------.
//       |               |            |          |          |            |    
//       Image <-------- Page        Region       Line      Word         Glyph
//            
//\endverbatim
/// 
/// 
/// <b>Root</b>: Abstract superclass of all classes in the SomeName
/// framework.
/// 
/// <b>Env</b> : A class providing a public interface to other
/// programs.
/// 
/// <b>Image</b> : A collection of pixels. In-memory representation of
/// a file in a known graphics format.
/// 
/// <b>Element</b> : Abstract superclass of a collection of classes
/// representing a part-whole decomposition of a graphic image into
/// smaller and smaller semantic units.
/// 
/// <b>Page</b> : An Image and associated meta-data representing an
/// entire page of to-be-scanned data. May be sub-divided into
/// Regions, Lines, Words and/or Glyphs, although only Glyphs are
/// crucial.
/// 
/// <b>Region</b> : A sub-region of a Page, usually used to represent
/// the graphical area for a single column in a multi-column image.
/// May be sub-divided into Lines, Words, and/or Glyphs. This class is
/// not currently prioritized and may be deleted if deemed
/// unnecessary.
/// 
/// <b>Line</b> : A sub-region of a Page (or Region) consisting
/// entirely of Glyphs. My be sub-divided into Words that contain
/// Glyphs, but may often contain only Glyphs.
/// 
/// <b>Word</b> : A sub-region of a Line consisting entirely of
/// Glyphs, separated from other Words within the Line by more
/// horizontal space than
/// 
/// <b>Glyph</b> : A sub-region of a Page (or Region or Line or Word)
/// representing a single to-be-identified character.
///
/// 
///
/// <hr>
/// \section strategy Strategy Class Hierarchy
///
/// Another hierarchy of classes has been added to the framework
/// to address an issue related to ease-of-extentions.
///
/// The above hierarchy defines some fundamental classes needed by the
/// OCR. There are various methods that can be defined with each of
/// those classes. For example, the Page class should contain a method
/// that processes the input image and produces text output, and could
/// thus defined a 'process()' method. This 'process()' method could
/// be divided into three sub-algorithms, 'partition()', 'identify()',
/// and 'format()', where the first identifies sub-regions of the page
/// representing Glyphs, the second converts Glyphs to unicode, and
/// the third positions characters relative to one another and
/// generates output. We assume those three methods are also defined
/// on Page.
///
/// Now, suppose a client of the framework wants to extend the
/// framework. They do this by creating subclasses of one or more of
/// the classes defined in the framework, and overriding one or more
/// methods within the newly introduced classes. For example, an
/// individual could create a MyPage class, and could redefine
/// 'partition() in order to write a better glyph partitioning
/// algorithm. This strategy is straight-forward and intuitive.
/// However, it sufferes from a serious drawback. In particular, there
/// are many possible implementations of the 'partition' algorithm,
/// many different implementations of the 'identify' algorithm, etc.
/// Suppose there are K variants for 'partition()', L variants of
/// 'identify' and M variants of 'format(). Using subclasses of Page,
/// the client would need to create K * L * M subclasses in order to
/// represent all combinations of the variations across these three
/// algorithms. Depending on the values of K, L and M, this number can
/// quickly become prohibitive, and that assumes there will only be 3
/// algorithms defined on Page. However, each of the methods
/// 'partition()', 'identify()' and 'format()' can be further divided
/// into many additional algorithms, which with many possible
/// implementations, dramatically increasing the number of subclasses
/// needed to represent all possible combinations.
///
/// There is, however, a simple way to be able to provide all possible
/// combinations of K * L * M items using only K + L + M classes. To
/// do so, we introduce a separate class hierarchy for each algorithm.
/// Each such hierarchy represents a single algorithm that the OCR
/// needs to perform, and each subclass provides a different strategy
/// for implementing that algorithm. Thus, there is a hierarchy of
/// classes that implement the 'partition' algorithm. Let's assume the
/// root of this hierarchy is called ElementitionStrategy. It can define
/// an abstract method called 'execute()' that all subclasses are
/// required to implemeent. One possible implementation of
/// partitioning is provided by the 'gocr' OCR, and we can create a
/// class called GocrElementition that subclasses ElementitionStrategy and
/// implements method 'execute()' by calling the appropriate 'gocr' C
/// methods. 
/// 
/// Similarily, we can define the abstract class IdentifyStrategy to
/// represent the glyph-to-unicode identification algorithm, and the
/// abstract class FormatStrategy to represent the overall
/// unicode-placement-and-output algorithm. Anyone can add a new
/// strategy for implementing any of these algorithms by creating a
/// subclass of the desired abstract class and implementing its
/// 'execute()' method.
///
/// Once these hierarchies have been defined, we can change the
/// implementation of the methods in Page. Before these hierarchies
/// were introduced, Page::partition() did partitioning itself, and
/// Page::identify() did identification itself, etc. Now, however,
/// each of these methods simply delegates execution to an instances
/// of some subclass of the appropriate Strategy class. For example,
/// if we want to use the GOCR algorithm for partitioning Glyphs, we
/// would have Page::partition() obtain an instance of GocrElementition,
/// and invoke 'execute()' on it (passing the Page as an argument).
/// 
/// A portion of the overall Strategy hierarchy is shown below.
///
/// \verbatim
//        
//                         Strategy                         
//                            |                             
//                            |                             
//         .-----------.------^-------.-----------.------------.----...
//         |           |              |           |            |
//   ProcessStrategy   |      IdentifyStrategy    |           ... 
//         |    ElementitionStrategy     |    FormatStrategy
//         |           |              |           |
//         |           |              |           |
//   StandardProcess   |        GocrIdentify      |
//                GocrElementition              GocrFormat
//
//
//            
// \endverbatim
///
/// The above design allows clients of the framework to selectively
/// contribute specific algorithms. And since the algorithms are
/// organized hierarchially (some algorithms are subdivided into a
/// sequence of calls to other algorithms) the client can control the
/// entire process, a very specific part of the process, or any
/// algorithm in between these two extremes.
///
/// However, the above design does not address how the Page class goes
/// about determining which subclass to use for each Strategy. We need
/// some means of allowing a Page to establish which Strategy subclass
/// to use for every formalized Strategy in the SomeName framework.
///
/// In considering solutions for this, not that sometimes the specific
/// solution for one problem (i.e. partitioning) is relatively
/// independent of the specific solution used for another problem
/// (i.e. formatting). However, in some situations, this may not be
/// true, and the use of a specific algorithm for one problem may
/// require the use of a specific algorithm for another problem. This
/// dependency between instances is conveniently addressed by the
/// AbstractFactory design pattern, which is responsible for
/// supporting the creation of families of products. However, our use
/// of AbstractFactory will be somewhat non-conventional, in that the
/// AbstractFactory will be both the maintainer of products, and the
/// creator of them (normally an AbstractFactory is created and passed
/// into a method in another class that uses the factory to create
/// products).
/// 
/// The overall idea is the following. We define a StrategyFactory
/// class that maintains a field for every algorithm we've formalized
/// within SomeName. Thus, for example, the class will have fields for
/// 'process', 'partition', 'identify', and 'format' (among many
/// others). Furthermore, the class defines one Factory Method for
/// each field. This factory method is responsible for creating an
/// instance of some subclass of the Strategy in question, and will be
/// used by StrategyFactory to initialize the associated field. For
/// example, StrategyFactory defines an 'identify' field of type
/// IdentifyStrategy*, and also defines a method called
/// 'createIdentifyStrategy()' that returns an IdentifyStrategy*.
///
/// The reason this design provides flexibility is that clients of the
/// framework can now subclass StrategyFactory to create a class that
/// explicitly specifies exactly which strategies are to be used for
/// every algorithm. Furthermore, this subclass can ensure that if
/// there are dependencies between algorithms, they will be enforced
/// (for example, by ensuring that any time strategy A1 for algorithm
/// A is used, strategy B1 for algorithm B is also used).
/// 
/// The following is a portion of the StrategyFactory hierarchy. It
/// will grow as new strategies are added to the framework.
/// 
/// \verbatim
//
//                     StrategyFactory
//                           |          
//         .-----------------^------------------.          
//         |                 |                  |
//        ???           GocrAlgorithms         ??? 
//
//
// \endverbatim
/// 
/// Note that the subclasses of StrategyFactory are very simplistic
/// consisting simply of a collection of 'create*Strategy()' methods
/// that create instances of specific subclasses of the appropriate
/// Strategy hierarchy. As such, we can provide tools that allow
/// clients to specify the algorithms to use without having to know
/// how to program at all. The tool will take the user-specified
/// information about which strategies to use for each algorithm and
/// automatically generate a subclass of StrategyFactory.
/// 
/// <hr>
/// \section terminology Terminology
/// 
///   - horizontal: the direction that aligns glyphs within a line.
///     Normally, this is just the x direction, but if the image is
///     rotated, then 'horizontal' refers to the x direction after
///     this rotation has been compensated for.
/// 
///   - vertical: the direction orthogonal to horizontal.
/// 
///   - line partitioning: The identification of the regions within a
///     Page representing individual Lines of horizontally adjacent
///     Glyphs. Line partitioning may occur before or after Glyph
///     partitioning, depending on the overall partitioning strategy
///     employed.
/// 
///   - glyph partitioning: The identification of the regions withiin
///     a Page representing individual Glyphs. Glyph partitioning is
///     not the same as Glyph identification, but instead preceeds it.
/// 
///   - glyph identification: The process of converting a graphic
///     image representing a character into a unicode value. The heart
///     of any OCR program. Its efficacy depends not only on its
///     internal strategies, but also the accuracy of Glyph
///     partitioning. If a region is identified as a Glyph, but is
///     instead only part of a glyph, or multiple glyphs, the accuracy
///     will naturally decline.
///
/// <hr>
/// \section problems Non-standard Implementation Issues
///
///  - The src/ directory contains all C++ classes (source and
///    header). The Makefile is designed to automatically maintain all
///    dependencies between files, and to ensure that a file is always
///    compiled if any file it depends on is modified. It is important
///    to note that the Makefile finds and compiles EVERY *.cc file in
///    the entire directory hierarchy (i.e. all sub-directories are
///    searched too). This means that you CANNOT create a "backup" or
///    "old" directory within 'src/' - such code will be compiled and
///    generate multiple-redefined symbol errors. Instead, backups and
///    old directories must be put outside the 'src' directory
///    hierarchy (usually in the parent directory of src, the root of
///    the SomeName directory hierarchy).
/// 
/// <hr>
/// \section guidelines Programming Guidelines
/// 
/// Readability, code-reuse, abstraction, modularity, encapsulation and
/// information hiding aren't just buzzwords, they are critical to smooth,
/// ongoing evolution of code. And they don't occur by magic. Here are
/// some guidelines to follow when working on this project.
/// 
///  - All fields are defined in private scope and are preceeded by an 
///    underscore.  Accessors are defined for each field.  See the
///    more detailed discussion on 'Accessors' below.
///     - getters can be public, but should ALWAYS have read-only 
///       semantics, and should be moved out of 'public' scope if the
///       field isn't critical to the public interface.
///     - setters should never be at public scope
/// 
///  - Classes should be fully initialized by their constructors. Elemential
///    initialization requires clients of the class to remember to perform
///    any additional initialization before using the class normally, and
///    completely defeats the point of having an initializer to begin
///    with. Furthermore, it requires setter methods to either be made
///    public (bad) or requires other classes to be added as friends (to
///    be avoided when possible). Full initialization within constructors
///    implies that classes should NOT have default constructors unless
///    the default values assigned to fields are not only intuitive and
///    meaningful, but also useable as-is.
///     - IMPORTANT: There is already one violation of the above mandate,
///       in StrategyFactory.
/// 
///  - Every class is documented. Every method is documented. Every field
///    is documented. Use '///' so that the class/method/field
///    documentation is available to doxygen. The UML diagram(s) are kept
///    up-to-date. Overall changes in design are kept up-to-date.
/// 
///  - Every class should have a static 'test()' method that acts as a
///    unit test. This method should create one or more instances of the
///    class (and any other classes needed to get the class to work) and
///    test the public interface methods, checking return results/output
///    against expected results to verify that it is working as expected.
/// 
///  - Avoid symbol conflicts. All classes are placed in the SomeName
///    namespace. No global variables. Do NOT place 'using namespace std'
///    statements in header files. Yes, I know it allows you to avoid
///    putting 'std::' in front of all sorts of types, but putting such a
///    statement in header files forces third-party code to fully-qualify
///    class names that conflict with those in 'std'. Using 'using
///    namespace std' in source files is completely ok.
/// 
///  - Avoid assuming there will only be one instances of any particular
///    class. Support for multi-threading/parallel/distributed algorithms
///    suggests that even classes that seem like Singletons may not be.
/// 
///  - Write code in a thread-safe manner.  Avoid static variables were
///    possible.
///
///  - Document all changes in the root ChangeLog file.  Especially
///    important to note is the introduction, changing or removal of
///    ANY new public interface methods in any class. Concise notes
///    any *everything* done is probably better than detailed notes on
///    a few things done. Detailed notes can be placed in appropriate
///    system/class/method documentation.
///
///  - All methods should be virtual by default.  It is unlikely
///    that the minor gains in efficiency from having a non-virtual
///    method will significantly improve performance. The exception to
///    this rule is setters and getters, which can incur a significant
///    efficiency hit if made virtual because they can then no longer
///    be inlined. Field accessors are normally not marked as virtual.
///
///  - If a method is virtual, explicitly specify the 'virtual' keyword
///    in the class header even when doing so is redundant (because it
///    was specified as virtual in a superclass). This convention
///    allows us to establish whether a method is virtual or not by
///    looking in a single header file, instead of having to search
///    every header file up the entire inheritance path when
///    virtualness is in doubt.
/// 
/// <hr>
/// \subsection accessors Accessors
/// 
/// Suppose we have a field called 'name', of type 'Type' (for example,
/// 'Type' might be 'int' or 'A*'). One way to define it would be:
/// 
/// \code
//   class A {
//     public:
//       Type name;
//   }  
//\endcode
/// 
/// And then an individual would read/write the field using:
///
/// \code
//   A* a = new A;
//   a->name = 10;
//   cout << "name = " << a->name << endl;
//\endcode
/// 
/// A public field is a terrible implementation decision because:
///   1) All code, everywhere, is allowed to modify (and thus potentially
///      corrupt) its value.  There is no "safety" available and
///      modification-at-a-distance is difficult to debug.
///   2) If, during the evolution of the code, the type of the field
///      (or the existence of the field) comes into question, changes
///      are both difficult, and guaranteed to break any third-party
///      code that relies on this field.
/// 
/// However, using a simple strategy can address both of these problems
/// and add some additional capability. And in C++, this comes at
/// absolutely no runtime performance cost, as we'll see below.
/// 
/// Instead of defining our field as shown above, we do the following:
/// 
/// \code
//   class A {
//     public:
//        inline const Type& name() const { return this->_name; }
//
//     protected:
//        inline void nameIs(Type val) { this->_name = val; }
//        inline Type& nameRef() { return this->_name; }
//
//     private:
//       Type _name;
//   }
//\endcode
/// 
/// In the above, we have moved the field definition itself into private
/// scope, and places an underscore in front of it.  We have also added
/// three tiny inlined methods providing get/set access to the field.
///
/// <hr>
/// \subsubsection features Features worth noting:
/// 
///   - The 'name()' method returns a const version of the underlying
///     field. It allows 'read-only' access to the field, but does not
///     allow it to be modified. It is often at public scope, but should
///     be placed in protected scope if the field is not part of the
///     public interface. Obviously, the fewer field accessors in the
///     public interface, the more separation between implementation and
///     interface.
/// 
///   - The 'nameIs()' method sets the value of the field to its single
///     argument. This method should almost never be placed at public
///     scope. Classes should be designed so that they, are a small set of
///     tightly-coupled friend classes, are responsible for modifying
///     themselves. In this way, corruption of state is much easier to
///     identify, because only a small number of classes can be
///     responsible, and ultimately the change had to have occured via
///     this setter method.
/// 
///   - The 'nameRef()' method has exactly the same code block as
///     'name()', but its return value and non-const status make it
///     much more powerful.  It too should almost never be placed at
///     public scope, as it provides a second means of modifying the
///     field.  For example, if Type is 'int', we can write:
/// \code
//        a->nameRef() = 10;
//\endcode
///     because 'nameRef()' returns a reference to the 'int' field. This
///     accessor is useful because anything we could do with the
///     underlying field 'a->_name', we can also do with 'a->nameRef()'.
///     For example:
/// \code
//         cout << a->_name << endl;
//         a->_name += 10;
//         ++a->_name;
//\endcode
///     is the same as
/// \code
//         cout << a->nameRef() << endl;
//         a->nameRef() += 10;
//         ++a->nameRef();
//\endcode
///     except that using 'nameRef' provides for some separation of
///     interface and implementation.  Note that since 'nameRef' is
///     inline, the above two sets of lines really are identical
///     after compilation.
/// 
/// <hr>
/// \subsubsection features2 Additional Features
/// 
///   - The type of the argument in 'nameIs()' can sometimes be made
///     'const', but when it is a pointer type, it cannot be made 'const'
///     unless the underlying field itself is 'const'.
/// 
///   - Although many C++ programmers avoid using 'const', constness
///     can have a significant positive impact on code quality.  Rather
///     than solving 'const' issues by not using 'const', it takes only
///     a little effort to understand the basic principles.  Remember that:
/// 
///       - If a variable is 'const', it cannot be modified. That is, if
///         it appears on the right-hand side of an assignment statement,
///         the compiler will generate an error.
///   
///       - 'const'ness is contagious - if one variable is 'const', it
///         often forces other variables or methods to also be 'const',
///         which may in turn force other variables or methods to be
///         'const', etc. etc. Because of this, the strategy of "I'll get
///         my code working without 'const' first, then add 'const'
///         semantics in later" is a terrible idea, because it is
///         difficult to "add in 'const'" incrementally - making one
///         variable/method 'const' requires many others to be made const
///         before the code will compile again.  Because of this, it is
///         much better to always make EVERYTHING (variables and methods)
///         'const' by default, at least conceptually, then remove
///         'const' when it is established that the variable/method in
///         question cannot be 'const' and still retain the desired
///         semantics.
/// 
///       - If a variable is 'const', then only 'const' methods can
///         be invoked with that variable as a receiver.  For example:
/// \code
//             const Person p;
//             p.somefunc();
// \endcode
///         Only if 'somefunc' is marked as 'const' in its definition:
/// \code
//             void somefunc() const { ... }
// \endcode
///         will the above 'p.somefunc();' invocation be allowed by
///         the compiler.
///       
///       - A non-static method f() defined within a class A has a
///         special variable called 'this'.  Since it is a variable,
///         it must have a type.  The type of 'this', within class A,
///         is ALWAYS either 'A* const' or 'const A*const'.  Which
///         one it is depends on whether the method f() is 'const'
///         or not.  
/// 
///          - A method is 'const' if the keyword 'const' appears
///            AFTER its parameter list.
/// 
///          - If 'this' is of type 'A* const', it means that the 
///            space for the pointer itself cannot be changed, so the
///            statement
/// \code
//                this = a;
// \endcode
///            is not allowed.  Since the 'this' variable is ALWAYS
///            'A* const' or 'const A*const', one is never allowed to
///            modify 'this'.  However, a variable being '*const'
///            does not affect the ability to modify what the variable
///            points to, so 
/// \code
//                this->some_field = 10;
// \endcode
///            isn't disallowed.
/// 
///          - If 'this' is of type 'const A* const', it means that
///            everything above for 'A* const' holds, AND that we
///            cannot modify what the variable points to, so
/// \code
//                this->some_field = 10;
// \endcode
///            becomes a compile-time error.
/// 

namespace SomeName {
  // ###################################################################
  /// \class Root
  ///
  /// \brief The abstract superclass of all classes in the SomeName
  /// namespace.
  ///
  /// Functionality defined here is available in every SomeName class,
  /// obviously. Primarily debugging and other low-level functionality.
  // ###################################################################

  class Root {
  public:
    // Constructors/Destructors
    Root();
    
    // Accessors
    
    // Input/Output
    
    // Miscellaneous
    
  protected:
    // Accessors
    
    // Methods 
    
  private: 
    // Accessors
    
    // Methods 
    
    // State
  };
};

# endif // Root_h

