# ifndef StrategyFactory_h
# define StrategyFactory_h StrategyFactory_h

# include "ProcessStrategy.h"
# include "SegmentStrategy.h"
# include "IdentifyStrategy.h"
# include "FormatStrategy.h"

namespace SomeName {

  /// \class StrategyFactory
  ///
  /// \brief AbstractFactory design pattern supporting the bundled
  /// creation of related Strategy algorithms.
  ///
  /// In order to make customization easy, SomeName identifies the
  /// fundamental algorithms necessary to perform OCR. Each of these
  /// algorithms is formalized into a Strategy class hierarchy -
  /// different subclasses provide different implementations of the
  /// algorithm. A particular run of the OCR establishes which
  /// strategy to use for every one of its algorithms, and performs
  /// them in the appropriate sequence (the sequence in which
  /// algorithms are applied is also one of the algorithms).
  ///
  /// Since the specific strategy for solving one problem might depend
  /// on or require a specific strategy for solving some other
  /// pattern, we introduce an Abstract Factory design pattern. In
  /// general, an Abstract Factory is a collection of Factory Methods
  /// (which produce products).  An Abstract Factory allows one to
  /// enforce dependencies between individual products created.
  ///
  /// NOTE: The products being produced here are instances of a
  /// specific subclass of each Strategy hierarchy. Strategy is
  /// another design pattern, and normally the subclasses are
  /// Singletons (the classes represent algorithms, and are normally
  /// stateless, or at least requiring only a single state). However,
  /// I have not yet fully established whether using Singletons will
  /// pose problems for multi-threading or parallel/distributed
  /// algorithms, so for now I'm not making Strategy subclasses
  /// singletons. If we do decide to do so, it might be easier to just
  /// have the StrategyFactory intern objects than to put the
  /// Singleton design pattern into every single Strategy subclass.
  /// There will be many Strategy hierarchies, and potentially many
  /// subclasses in each hierarchy.

  class StrategyFactory {
  public:
    // Constructors/Destructors
    StrategyFactory();
    
    // Accessors
    //  - This class handles accessors for its algorithms differently than
    //    normal. The strategy is return non-const, and no 'xxxRef()'
    //    method is defined. This is because other classes always need
    //    non-const access to these algorithms in order to function.
    inline FormatStrategy*          format()    const { return this->_format; }
    inline IdentifyStrategy*        identify()  const { return this->_identify; }
    inline SegmentStrategy*         segment()   const { return this->_segment; }
    inline ProcessStrategy*         process()   const { return this->_process; }
    
    void init();
    
    // Miscellaneous
    virtual ProcessStrategy*        createProcessStrategy()   const = 0;
    virtual SegmentStrategy*        createSegmentStrategy() const = 0;
    virtual IdentifyStrategy*       createIdentifyStrategy()  const = 0;
    virtual FormatStrategy*         createFormatStrategy()    const = 0;

  protected:
    // Accessors
    inline void                     formatIs(FormatStrategy* format) { this->_format = format; }
    inline void                     identifyIs(IdentifyStrategy* identify) { this->_identify = identify; }
    inline void                     segmentIs(SegmentStrategy* segment) { this->_segment = segment; }
    inline void                     processIs(ProcessStrategy* process) { this->_process = process; }
    
    // Methods 
    
  private: 
    // Accessors

    // Methods 

    // State
    FormatStrategy*                 _format;
    IdentifyStrategy*               _identify;
    SegmentStrategy*                _segment;
    ProcessStrategy*                _process;
  };
};

# endif // StrategyFactory_h

